@@ -30,10 +30,10 @@ class LDAbase : virtual public keyATMmeta
     // In LDA, we do not need to read and initialize X
     virtual void read_data_common();
     virtual void initialize_common();
-    virtual void iteration_single(int &it) = 0;
-    void parameters_store(int &r_index); 
-    virtual int sample_z(VectorXd &alpha, int &z, int &s,
-                         int &w, int &doc_id) final;
+    virtual void iteration_single(int it) = 0;
+    void parameters_store(int r_index); 
+    virtual int sample_z(VectorXd &alpha, int z, int s,
+                         int w, int doc_id) final;
     virtual double loglik_total() = 0;
 };
 
